]> nv-tegra.nvidia Code Review - linux-2.6.git/blobdiff - ipc/sem.c
Remove obsolete #include <linux/config.h>
[linux-2.6.git] / ipc / sem.c
index 31fd4027d2b5bd7dbda1ad88a69a6460a6af920c..6013c751156fe81bee30536bef27bab7792eb1bb 100644 (file)
--- a/ipc/sem.c
+++ b/ipc/sem.c
  * (c) 2001 Red Hat Inc <alan@redhat.com>
  * Lockless wakeup
  * (c) 2003 Manfred Spraul <manfred@colorfullife.com>
+ *
+ * support for audit of ipc object properties and permission changes
+ * Dustin Kirkland <dustin.kirkland@us.ibm.com>
  */
 
-#include <linux/config.h>
 #include <linux/slab.h>
 #include <linux/spinlock.h>
 #include <linux/init.h>
@@ -75,6 +77,8 @@
 #include <linux/audit.h>
 #include <linux/capability.h>
 #include <linux/seq_file.h>
+#include <linux/mutex.h>
+
 #include <asm/uaccess.h>
 #include "util.h"
 
@@ -139,7 +143,7 @@ void __init sem_init (void)
  *     * if it's IN_WAKEUP, then it must wait until the value changes
  *     * if it's not -EINTR, then the operation was completed by
  *       update_queue. semtimedop can return queue.status without
- *       performing any operation on the semaphore array.
+ *       performing any operation on the sem array.
  *     * otherwise it must acquire the spinlock and check what's up.
  *
  * The two-stage algorithm is necessary to protect against the following
@@ -214,7 +218,7 @@ asmlinkage long sys_semget (key_t key, int nsems, int semflg)
 
        if (nsems < 0 || nsems > sc_semmsl)
                return -EINVAL;
-       down(&sem_ids.sem);
+       mutex_lock(&sem_ids.mutex);
        
        if (key == IPC_PRIVATE) {
                err = newary(key, nsems, semflg);
@@ -227,8 +231,7 @@ asmlinkage long sys_semget (key_t key, int nsems, int semflg)
                err = -EEXIST;
        } else {
                sma = sem_lock(id);
-               if(sma==NULL)
-                       BUG();
+               BUG_ON(sma==NULL);
                if (nsems > sma->sem_nsems)
                        err = -EINVAL;
                else if (ipcperms(&sma->sem_perm, semflg))
@@ -242,7 +245,7 @@ asmlinkage long sys_semget (key_t key, int nsems, int semflg)
                sem_unlock(sma);
        }
 
-       up(&sem_ids.sem);
+       mutex_unlock(&sem_ids.mutex);
        return err;
 }
 
@@ -437,8 +440,8 @@ static int count_semzcnt (struct sem_array * sma, ushort semnum)
        return semzcnt;
 }
 
/* Free a semaphore set. freeary() is called with sem_ids.sem down and
* the spinlock for this semaphore set hold. sem_ids.sem remains locked
  * on exit.
  */
 static void freeary (struct sem_array *sma, int id)
@@ -525,7 +528,7 @@ static int semctl_nolock(int semid, int semnum, int cmd, int version, union semu
                seminfo.semmnu = SEMMNU;
                seminfo.semmap = SEMMAP;
                seminfo.semume = SEMUME;
-               down(&sem_ids.sem);
+               mutex_lock(&sem_ids.mutex);
                if (cmd == SEM_INFO) {
                        seminfo.semusz = sem_ids.in_use;
                        seminfo.semaem = used_sems;
@@ -534,7 +537,7 @@ static int semctl_nolock(int semid, int semnum, int cmd, int version, union semu
                        seminfo.semaem = SEMAEM;
                }
                max_id = sem_ids.max_id;
-               up(&sem_ids.sem);
+               mutex_unlock(&sem_ids.mutex);
                if (copy_to_user (arg.__buf, &seminfo, sizeof(struct seminfo))) 
                        return -EFAULT;
                return (max_id < 0) ? 0: max_id;
@@ -809,8 +812,6 @@ static int semctl_down(int semid, int semnum, int cmd, int version, union semun
        if(cmd == IPC_SET) {
                if(copy_semid_from_user (&setbuf, arg.buf, version))
                        return -EFAULT;
-               if ((err = audit_ipc_perms(0, setbuf.uid, setbuf.gid, setbuf.mode)))
-                       return err;
        }
        sma = sem_lock(semid);
        if(sma==NULL)
@@ -821,7 +822,16 @@ static int semctl_down(int semid, int semnum, int cmd, int version, union semun
                goto out_unlock;
        }       
        ipcp = &sma->sem_perm;
-       
+
+       err = audit_ipc_obj(ipcp);
+       if (err)
+               goto out_unlock;
+
+       if (cmd == IPC_SET) {
+               err = audit_ipc_set_perm(0, setbuf.uid, setbuf.gid, setbuf.mode);
+               if (err)
+                       goto out_unlock;
+       }
        if (current->euid != ipcp->cuid && 
            current->euid != ipcp->uid && !capable(CAP_SYS_ADMIN)) {
                err=-EPERM;
@@ -886,9 +896,9 @@ asmlinkage long sys_semctl (int semid, int semnum, int cmd, union semun arg)
                return err;
        case IPC_RMID:
        case IPC_SET:
-               down(&sem_ids.sem);
+               mutex_lock(&sem_ids.mutex);
                err = semctl_down(semid,semnum,cmd,version,arg);
-               up(&sem_ids.sem);
+               mutex_unlock(&sem_ids.mutex);
                return err;
        default:
                return -EINVAL;
@@ -1182,8 +1192,7 @@ retry_undos:
 
        sma = sem_lock(semid);
        if(sma==NULL) {
-               if(queue.prev != NULL)
-                       BUG();
+               BUG_ON(queue.prev != NULL);
                error = -EIDRM;
                goto out_free;
        }
@@ -1300,9 +1309,9 @@ found:
                /* perform adjustments registered in u */
                nsems = sma->sem_nsems;
                for (i = 0; i < nsems; i++) {
-                       struct sem * sem = &sma->sem_base[i];
+                       struct sem * semaphore = &sma->sem_base[i];
                        if (u->semadj[i]) {
-                               sem->semval += u->semadj[i];
+                               semaphore->semval += u->semadj[i];
                                /*
                                 * Range checks of the new semaphore value,
                                 * not defined by sus:
@@ -1316,11 +1325,11 @@ found:
                                 *
                                 *      Manfred <manfred@colorfullife.com>
                                 */
-                               if (sem->semval < 0)
-                                       sem->semval = 0;
-                               if (sem->semval > SEMVMX)
-                                       sem->semval = SEMVMX;
-                               sem->sempid = current->tgid;
+                               if (semaphore->semval < 0)
+                                       semaphore->semval = 0;
+                               if (semaphore->semval > SEMVMX)
+                                       semaphore->semval = SEMVMX;
+                               semaphore->sempid = current->tgid;
                        }
                }
                sma->sem_otime = get_seconds();
@@ -437,8 +440,8 @@ static int count_semzcnt (struct sem_array * sma, ushort semnum)
 }
 
-/* Free a semaphore set. freeary() is called with sem_ids.sem down and
- * the spinlock for this semaphore set hold. sem_ids.sem remains locked
+/* Free a semaphore set. freeary() is called with sem_ids.mutex locked and
+ * the spinlock for this semaphore set hold. sem_ids.mutex remains locked
  * on exit.
  */